@@ -26,12 +26,12 @@
 #ifndef _COMPIZ_H
 #define _COMPIZ_H
 
-#define ABIVERSION 20070708
+#define ABIVERSION 20070815
 
 #include <stdio.h>
 #include <sys/time.h>
 
-#include <X11/Xlib.h>
+#include <X11/Xlib-xcb.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xcomposite.h>
@@ -278,6 +278,10 @@ typedef enum {
     CompOptionTypeString,
     CompOptionTypeColor,
     CompOptionTypeAction,
+    CompOptionTypeKey,
+    CompOptionTypeButton,
+    CompOptionTypeEdge,
+    CompOptionTypeBell,
     CompOptionTypeMatch,
     CompOptionTypeList
 } CompOptionType;
@@ -310,7 +314,6 @@ typedef enum {
     CompLogLevelWarn,
     CompLogLevelInfo,
     CompLogLevelDebug
-
 } CompLogLevel;
 
 typedef struct _CompKeyBinding {
@@ -347,7 +350,6 @@ struct _CompAction {
     Bool bell;
 
     unsigned int edgeMask;
-    int		 edgeButton;
 
     CompPrivate priv;
 };
@@ -403,6 +405,13 @@ struct _CompOption {
 typedef CompOption *(*DisplayOptionsProc) (CompDisplay *display, int *count);
 typedef CompOption *(*ScreenOptionsProc) (CompScreen *screen, int *count);
 
+void
+compInitOptionValue (CompOptionValue *v);
+
+void
+compFiniOptionValue (CompOptionValue *v,
+		     CompOptionType  type);
+
 void
 compInitOption (CompOption *option);
 
@@ -412,7 +421,7 @@ compFiniOption (CompOption *option);
 CompOption *
 compFindOption (CompOption *option,
 		int	    nOption,
-		char	    *name,
+		const char  *name,
 		int	    *index);
 
 Bool
@@ -457,37 +466,37 @@ compWindowTypeMaskFromStringList (CompOptionValue *value);
 Bool
 getBoolOptionNamed (CompOption *option,
 		    int	       nOption,
-		    char       *name,
+		    const char *name,
 		    Bool       defaultValue);
 
 int
 getIntOptionNamed (CompOption *option,
 		   int	      nOption,
-		   char	      *name,
+		   const char *name,
 		   int	      defaultValue);
 
 float
 getFloatOptionNamed (CompOption *option,
 		     int	nOption,
-		     char	*name,
+		     const char *name,
 		     float	defaultValue);
 
 char *
 getStringOptionNamed (CompOption *option,
 		      int	 nOption,
-		      char	 *name,
+		      const char *name,
 		      char	 *defaultValue);
 
 unsigned short *
 getColorOptionNamed (CompOption	    *option,
 		     int	    nOption,
-		     char	    *name,
+		     const char     *name,
 		     unsigned short *defaultValue);
 
 CompMatch *
 getMatchOptionNamed (CompOption	*option,
 		     int	nOption,
-		     char	*name,
+		     const char *name,
 		     CompMatch  *defaultValue);
 
 char *
@@ -498,6 +507,14 @@ char *
 buttonBindingToString (CompDisplay       *d,
 		       CompButtonBinding *button);
 
+char *
+keyActionToString (CompDisplay *d,
+		   CompAction  *action);
+
+char *
+buttonActionToString (CompDisplay *d,
+		      CompAction  *action);
+
 Bool
 stringToKeyBinding (CompDisplay    *d,
 		    const char     *binding,
@@ -508,9 +525,25 @@ stringToButtonBinding (CompDisplay	 *d,
 		       const char	 *binding,
 		       CompButtonBinding *button);
 
-char *
+void
+stringToKeyAction (CompDisplay *d,
+		   const char  *binding,
+		   CompAction  *action);
+
+void
+stringToButtonAction (CompDisplay *d,
+		      const char  *binding,
+		      CompAction  *action);
+
+const char *
 edgeToString (unsigned int edge);
 
+unsigned int
+stringToEdgeMask (const char *edge);
+
+char *
+edgeMaskToString (unsigned int edgeMask);
+
 Bool
 stringToColor (const char     *color,
 	       unsigned short *rgba);
@@ -518,9 +551,12 @@ stringToColor (const char     *color,
 char *
 colorToString (unsigned short *rgba);
 
-char *
+const char *
 optionTypeToString (CompOptionType type);
 
+Bool
+isActionOption (CompOption *option);
+
 
 /* display.c */
 
@@ -528,72 +564,73 @@ typedef int CompTimeoutHandle;
 typedef int CompWatchFdHandle;
 typedef int CompFileWatchHandle;
 
-#define COMP_DISPLAY_OPTION_ACTIVE_PLUGINS                0
-#define COMP_DISPLAY_OPTION_TEXTURE_FILTER                1
-#define COMP_DISPLAY_OPTION_CLICK_TO_FOCUS                2
-#define COMP_DISPLAY_OPTION_AUTORAISE                     3
-#define COMP_DISPLAY_OPTION_AUTORAISE_DELAY               4
-#define COMP_DISPLAY_OPTION_CLOSE_WINDOW                  5
-#define COMP_DISPLAY_OPTION_MAIN_MENU                     6
-#define COMP_DISPLAY_OPTION_RUN_DIALOG                    7
-#define COMP_DISPLAY_OPTION_COMMAND0                      8
-#define COMP_DISPLAY_OPTION_COMMAND1                      9
-#define COMP_DISPLAY_OPTION_COMMAND2                      10
-#define COMP_DISPLAY_OPTION_COMMAND3                      11
-#define COMP_DISPLAY_OPTION_COMMAND4                      12
-#define COMP_DISPLAY_OPTION_COMMAND5                      13
-#define COMP_DISPLAY_OPTION_COMMAND6                      14
-#define COMP_DISPLAY_OPTION_COMMAND7                      15
-#define COMP_DISPLAY_OPTION_COMMAND8                      16
-#define COMP_DISPLAY_OPTION_COMMAND9                      17
-#define COMP_DISPLAY_OPTION_COMMAND10                     18
-#define COMP_DISPLAY_OPTION_COMMAND11                     19
-#define COMP_DISPLAY_OPTION_RUN_COMMAND0                  20
-#define COMP_DISPLAY_OPTION_RUN_COMMAND1                  21
-#define COMP_DISPLAY_OPTION_RUN_COMMAND2                  22
-#define COMP_DISPLAY_OPTION_RUN_COMMAND3                  23
-#define COMP_DISPLAY_OPTION_RUN_COMMAND4                  24
-#define COMP_DISPLAY_OPTION_RUN_COMMAND5                  25
-#define COMP_DISPLAY_OPTION_RUN_COMMAND6                  26
-#define COMP_DISPLAY_OPTION_RUN_COMMAND7                  27
-#define COMP_DISPLAY_OPTION_RUN_COMMAND8                  28
-#define COMP_DISPLAY_OPTION_RUN_COMMAND9                  29
-#define COMP_DISPLAY_OPTION_RUN_COMMAND10                 30
-#define COMP_DISPLAY_OPTION_RUN_COMMAND11                 31
-#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS               32
-#define COMP_DISPLAY_OPTION_RAISE_WINDOW                  33
-#define COMP_DISPLAY_OPTION_LOWER_WINDOW                  34
-#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW             35
-#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW               36
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW               37
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ          38
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT          39
-#define COMP_DISPLAY_OPTION_OPACITY_INCREASE              40
-#define COMP_DISPLAY_OPTION_OPACITY_DECREASE              41
-#define COMP_DISPLAY_OPTION_SCREENSHOT                    42
-#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT                43
-#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT             44
-#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT         45
-#define COMP_DISPLAY_OPTION_WINDOW_MENU                   46
-#define COMP_DISPLAY_OPTION_SHOW_DESKTOP                  47
-#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                48
-#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                  49
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED       50
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ  51
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT  52
-#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS     53
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED          54
-#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED   55
-#define COMP_DISPLAY_OPTION_TERMINAL			  56
-#define COMP_DISPLAY_OPTION_RUN_TERMINAL		  57
-#define COMP_DISPLAY_OPTION_PING_DELAY			  58
-#define COMP_DISPLAY_OPTION_NUM				  59
+#define COMP_DISPLAY_OPTION_ACTIVE_PLUGINS                   0
+#define COMP_DISPLAY_OPTION_TEXTURE_FILTER                   1
+#define COMP_DISPLAY_OPTION_CLICK_TO_FOCUS                   2
+#define COMP_DISPLAY_OPTION_AUTORAISE                        3
+#define COMP_DISPLAY_OPTION_AUTORAISE_DELAY                  4
+#define COMP_DISPLAY_OPTION_CLOSE_WINDOW_KEY                 5
+#define COMP_DISPLAY_OPTION_MAIN_MENU_KEY                    6
+#define COMP_DISPLAY_OPTION_RUN_DIALOG_KEY                   7
+#define COMP_DISPLAY_OPTION_COMMAND0                         8
+#define COMP_DISPLAY_OPTION_COMMAND1                         9
+#define COMP_DISPLAY_OPTION_COMMAND2                         10
+#define COMP_DISPLAY_OPTION_COMMAND3                         11
+#define COMP_DISPLAY_OPTION_COMMAND4                         12
+#define COMP_DISPLAY_OPTION_COMMAND5                         13
+#define COMP_DISPLAY_OPTION_COMMAND6                         14
+#define COMP_DISPLAY_OPTION_COMMAND7                         15
+#define COMP_DISPLAY_OPTION_COMMAND8                         16
+#define COMP_DISPLAY_OPTION_COMMAND9                         17
+#define COMP_DISPLAY_OPTION_COMMAND10                        18
+#define COMP_DISPLAY_OPTION_COMMAND11                        19
+#define COMP_DISPLAY_OPTION_RUN_COMMAND0_KEY                 20
+#define COMP_DISPLAY_OPTION_RUN_COMMAND1_KEY                 21
+#define COMP_DISPLAY_OPTION_RUN_COMMAND2_KEY                 22
+#define COMP_DISPLAY_OPTION_RUN_COMMAND3_KEY                 23
+#define COMP_DISPLAY_OPTION_RUN_COMMAND4_KEY                 24
+#define COMP_DISPLAY_OPTION_RUN_COMMAND5_KEY                 25
+#define COMP_DISPLAY_OPTION_RUN_COMMAND6_KEY                 26
+#define COMP_DISPLAY_OPTION_RUN_COMMAND7_KEY                 27
+#define COMP_DISPLAY_OPTION_RUN_COMMAND8_KEY                 28
+#define COMP_DISPLAY_OPTION_RUN_COMMAND9_KEY                 29
+#define COMP_DISPLAY_OPTION_RUN_COMMAND10_KEY                30
+#define COMP_DISPLAY_OPTION_RUN_COMMAND11_KEY                31
+#define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS_KEY              32
+#define COMP_DISPLAY_OPTION_RAISE_WINDOW_BUTTON              33
+#define COMP_DISPLAY_OPTION_LOWER_WINDOW_BUTTON              34
+#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW_KEY            35
+#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW_KEY              36
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_KEY              37
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ_KEY         38
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT_KEY         39
+#define COMP_DISPLAY_OPTION_OPACITY_INCREASE_BUTTON          40
+#define COMP_DISPLAY_OPTION_OPACITY_DECREASE_BUTTON          41
+#define COMP_DISPLAY_OPTION_SCREENSHOT                       42
+#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT_KEY               43
+#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT                44
+#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT_KEY        45
+#define COMP_DISPLAY_OPTION_WINDOW_MENU_BUTTON               46
+#define COMP_DISPLAY_OPTION_WINDOW_MENU_KEY                  47
+#define COMP_DISPLAY_OPTION_SHOW_DESKTOP_KEY                 48
+#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                   49
+#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                     50
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_KEY      51
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ_KEY 52
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT_KEY 53
+#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS        54
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED_KEY         55
+#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED      56
+#define COMP_DISPLAY_OPTION_TERMINAL			     57
+#define COMP_DISPLAY_OPTION_RUN_TERMINAL_KEY		     58
+#define COMP_DISPLAY_OPTION_PING_DELAY			     59
+#define COMP_DISPLAY_OPTION_NUM				     60
 
 typedef CompOption *(*GetDisplayOptionsProc) (CompDisplay *display,
 					      int	  *count);
 
 typedef Bool (*SetDisplayOptionProc) (CompDisplay     *display,
-				      char	      *name,
+				      const char      *name,
 				      CompOptionValue *value);
 
 typedef CompOption *(*GetPluginDisplayOptionsProc) (CompPlugin  *plugin,
@@ -602,12 +639,12 @@ typedef CompOption *(*GetPluginDisplayOptionsProc) (CompPlugin  *plugin,
 
 typedef Bool (*SetPluginDisplayOptionProc) (CompPlugin      *plugin,
 					    CompDisplay     *display,
-					    char	    *name,
+					    const char	    *name,
 					    CompOptionValue *value);
 
 typedef Bool (*SetDisplayOptionForPluginProc) (CompDisplay     *display,
-					       char	       *plugin,
-					       char	       *name,
+					       const char      *plugin,
+					       const char	*name,
 					       CompOptionValue *value);
 
 typedef Bool (*InitPluginForDisplayProc) (CompPlugin  *plugin,
@@ -620,8 +657,8 @@ typedef void (*HandleEventProc) (CompDisplay *display,
 				 XEvent	     *event);
 
 typedef void (*HandleCompizEventProc) (CompDisplay *display,
-				       char        *pluginName,
-				       char        *eventName,
+				       const char  *pluginName,
+				       const char  *eventName,
 				       CompOption  *option,
 				       int         nOption);
 
@@ -727,11 +764,13 @@ typedef void (*MatchPropertyChangedProc) (CompDisplay *display,
 					  CompWindow  *window);
 
 typedef void (*LogMessageProc) (CompDisplay  *d,
-				char         *componentName,
+				const char   *componentName,
 				CompLogLevel level,
-				char         *message);
+				const char   *message);
 
 struct _CompDisplay {
+    xcb_connection_t *connection;
+
     Display    *display;
     CompScreen *screens;
 
@@ -989,16 +1028,16 @@ fileWatchRemoved (CompDisplay   *display,
 
 void
 compLogMessage (CompDisplay  *d,
-		char         *componentName,
+		const char   *componentName,
 		CompLogLevel level,
-		char         *format,
+		const char   *format,
 		...);
 
 void
 logMessage (CompDisplay  *d,
-	    char         *componentName,
+	    const char   *componentName,
 	    CompLogLevel level,
-	    char         *message);
+	    const char   *message);
 
 const char *
 logLevelToString (CompLogLevel level);
@@ -1011,7 +1050,7 @@ addScreenToDisplay (CompDisplay *display,
 		    CompScreen *s);
 
 Bool
-addDisplay (char *name);
+addDisplay (const char *name);
 
 Time
 getCurrentTimeFromDisplay (CompDisplay *d);
@@ -1115,8 +1154,8 @@ handleEvent (CompDisplay *display,
 
 void
 handleCompizEvent (CompDisplay *display,
-		   char        *pluginName,
-		   char        *eventName,
+		   const char  *pluginName,
+		   const char  *eventName,
 		   CompOption  *option,
 		   int         nOption);
 
@@ -1457,14 +1496,14 @@ destroyTexture (CompScreen  *screen,
 Bool
 imageBufferToTexture (CompScreen   *screen,
 		      CompTexture  *texture,
-		      char	   *image,
+		      const char   *image,
 		      unsigned int width,
 		      unsigned int height);
 
 Bool
 imageDataToTexture (CompScreen   *screen,
 		    CompTexture  *texture,
-		    char	 *image,
+		    const char	 *image,
 		    unsigned int width,
 		    unsigned int height,
 		    GLenum       format,
@@ -1636,18 +1675,18 @@ typedef void (*GLGenerateMipmapProc) (GLenum target);
 typedef CompOption *(*GetScreenOptionsProc) (CompScreen *screen,
 					     int	*count);
 typedef Bool (*SetScreenOptionProc) (CompScreen      *screen,
-				     char	     *name,
+				     const char	     *name,
 				     CompOptionValue *value);
 typedef CompOption *(*GetPluginScreenOptionsProc) (CompPlugin *plugin,
 						   CompScreen *screen,
 						   int	      *count);
 typedef Bool (*SetPluginScreenOptionProc) (CompPlugin      *plugin,
 					   CompScreen      *screen,
-					   char	           *name,
+					   const char	   *name,
 					   CompOptionValue *value);
 typedef Bool (*SetScreenOptionForPluginProc) (CompScreen      *screen,
-					      char	      *plugin,
-					      char	      *name,
+					      const char      *plugin,
+					      const char      *name,
 					      CompOptionValue *value);
 
 typedef Bool (*InitPluginForScreenProc) (CompPlugin *plugin,
@@ -2830,7 +2869,7 @@ typedef void (*FiniPluginProc) (CompPlugin *plugin);
 typedef CompMetadata *(*GetMetadataProc) (CompPlugin *plugin);
 
 typedef struct _CompPluginVTable {
-    char *name;
+    const char *name;
 
     GetVersionProc  getVersion;
     GetMetadataProc getMetadata;
@@ -2856,13 +2895,13 @@ typedef struct _CompPluginVTable {
 typedef CompPluginVTable *(*PluginGetInfoProc) (void);
 
 typedef Bool (*LoadPluginProc) (CompPlugin *p,
-				char	   *path,
-				char	   *name);
+				const char *path,
+				const char *name);
 
 typedef void (*UnloadPluginProc) (CompPlugin *p);
 
-typedef char **(*ListPluginsProc) (char *path,
-				   int	*n);
+typedef char **(*ListPluginsProc) (const char *path,
+				   int	      *n);
 
 extern LoadPluginProc   loaderLoadPlugin;
 extern UnloadPluginProc loaderUnloadPlugin;
@@ -2907,10 +2946,10 @@ void
 windowFiniPlugins (CompWindow *w);
 
 CompPlugin *
-findActivePlugin (char *name);
+findActivePlugin (const char *name);
 
 CompPlugin *
-loadPlugin (char *plugin);
+loadPlugin (const char *plugin);
 
 void
 unloadPlugin (CompPlugin *p);
@@ -2959,15 +2998,15 @@ destroyFunctionData (CompFunctionData *data);
 
 Bool
 addTempHeaderOpToFunctionData (CompFunctionData *data,
-			       char		*name);
+			       const char	*name);
 
 Bool
 addParamHeaderOpToFunctionData (CompFunctionData *data,
-				char		 *name);
+				const char	 *name);
 
 Bool
 addAttribHeaderOpToFunctionData (CompFunctionData *data,
-				 char		  *name);
+				 const char	  *name);
 
 #define COMP_FETCH_TARGET_2D   0
 #define COMP_FETCH_TARGET_RECT 1
@@ -2975,28 +3014,28 @@ addAttribHeaderOpToFunctionData (CompFunctionData *data,
 
 Bool
 addFetchOpToFunctionData (CompFunctionData *data,
-			  char		   *dst,
-			  char		   *offset,
+			  const char	   *dst,
+			  const char	   *offset,
 			  int		   target);
 
 Bool
 addColorOpToFunctionData (CompFunctionData *data,
-			  char		   *dst,
-			  char		   *src);
+			  const char	   *dst,
+			  const char	   *src);
 
 Bool
 addDataOpToFunctionData (CompFunctionData *data,
-			 char		  *str,
+			 const char	  *str,
 			 ...);
 
 Bool
 addBlendOpToFunctionData (CompFunctionData *data,
-			  char		   *str,
+			  const char	   *str,
 			  ...);
 
 int
 createFragmentFunction (CompScreen	 *s,
-			char		 *name,
+			const char	 *name,
 			CompFunctionData *data);
 
 void
