@@ -33,12 +33,12 @@
 #include <math.h>
 #include <unistd.h>
 
-#include <X11/Xatom.h>
-#include <X11/extensions/shape.h>
-
 #include <compiz.h>
 #include <decoration.h>
 
+#include <X11/Xatom.h>
+#include <X11/extensions/shape.h>
+
 static CompMetadata decorMetadata;
 
 typedef struct _Vector {
@@ -1115,7 +1115,7 @@ decorGetDisplayOptions (CompPlugin  *plugin,
 static Bool
 decorSetDisplayOption (CompPlugin      *plugin,
 		       CompDisplay     *display,
-		       char	       *name,
+		       const char      *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
