@@ -125,7 +125,7 @@ fadeGetScreenOptions (CompPlugin *plugin,
 static Bool
 fadeSetScreenOption (CompPlugin      *plugin,
 		     CompScreen      *screen,
-		     char	     *name,
+		     const char	     *name,
 		     CompOptionValue *value)
 {
     CompOption *o;
