@@ -37,7 +37,7 @@
 
 static CompMetadata gconfMetadata;
 
-#define APP_NAME "/apps/compiz"
+#define APP_NAME "compiz"
 
 /* From gconf-internal.h. Bleah. */
 int gconf_value_compare (const GConfValue *value_a,
@@ -48,11 +48,13 @@ static int displayPrivateIndex;
 typedef struct _GConfDisplay {
     int screenPrivateIndex;
 
-    GConfClient *client;
-
     InitPluginForDisplayProc      initPluginForDisplay;
     SetDisplayOptionProc	  setDisplayOption;
     SetDisplayOptionForPluginProc setDisplayOptionForPlugin;
+
+    GConfClient *client;
+
+    CompTimeoutHandle reloadHandle;
 } GConfDisplay;
 
 typedef struct _GConfScreen {
@@ -73,36 +75,22 @@ typedef struct _GConfScreen {
 #define GCONF_SCREEN(s)						           \
     GConfScreen *gs = GET_GCONF_SCREEN (s, GET_GCONF_DISPLAY (s->display))
 
-static int
-strcmpskipifequal (char **ptr,
-		   char *s)
-{
-    int ret, len;
-
-    len = strlen (s);
-    ret = strncmp (*ptr, s, len);
-    if (ret == 0)
-	*ptr = (*ptr) + len;
-
-    return ret;
-}
-
 static GConfValueType
 gconfTypeFromCompType (CompOptionType type)
 {
     switch (type) {
     case CompOptionTypeBool:
+    case CompOptionTypeBell:
 	return GCONF_VALUE_BOOL;
     case CompOptionTypeInt:
 	return GCONF_VALUE_INT;
     case CompOptionTypeFloat:
 	return GCONF_VALUE_FLOAT;
     case CompOptionTypeString:
-	return GCONF_VALUE_STRING;
     case CompOptionTypeColor:
-	return GCONF_VALUE_STRING;
-    case CompOptionTypeAction:
-	return GCONF_VALUE_STRING;
+    case CompOptionTypeKey:
+    case CompOptionTypeButton:
+    case CompOptionTypeEdge:
     case CompOptionTypeMatch:
 	return GCONF_VALUE_STRING;
     case CompOptionTypeList:
@@ -141,6 +129,33 @@ gconfSetValue (CompDisplay     *d,
 
 	free (color);
     } break;
+    case CompOptionTypeKey: {
+	gchar *action;
+
+	action = keyActionToString (d, &value->action);
+	gconf_value_set_string (gvalue, action);
+
+	free (action);
+    } break;
+    case CompOptionTypeButton: {
+	gchar *action;
+
+	action = buttonActionToString (d, &value->action);
+	gconf_value_set_string (gvalue, action);
+
+	free (action);
+    } break;
+    case CompOptionTypeEdge: {
+	gchar *edge;
+
+	edge = edgeMaskToString (value->action.edgeMask);
+	gconf_value_set_string (gvalue, edge);
+
+	free (edge);
+    } break;
+    case CompOptionTypeBell:
+	gconf_value_set_bool (gvalue, value->action.bell);
+	break;
     case CompOptionTypeMatch: {
 	gchar *match;
 
@@ -154,173 +169,37 @@ gconfSetValue (CompDisplay     *d,
     }
 }
 
-static void
-gconfSetActionValue (CompDisplay     *d,
-		     CompOptionValue *value,
-		     CompOptionType  type,
-		     GConfValue      *gvalue,
-		     CompBindingType bindingType)
-{
-    char *binding = NULL;
-
-    if (bindingType == CompBindingTypeKey)
-    {
-	if (value->action.type & CompBindingTypeKey)
-	    binding = keyBindingToString (d, &value->action.key);
-    }
-    else
-    {
-	if (value->action.type & CompBindingTypeButton)
-	    binding = buttonBindingToString (d, &value->action.button);
-    }
-
-    if (!binding)
-	binding = strdup ("Disabled");
-
-    gconf_value_set_string (gvalue, binding);
-
-    free (binding);
-}
-
 static void
 gconfSetOption (CompDisplay *d,
 		CompOption  *o,
-		gchar	    *screen,
-		gchar	    *plugin)
+		const gchar *plugin,
+		const gchar *object)
 {
-    GConfValue *gvalue, *existingValue = NULL;
-    gchar      *key;
+    GConfValueType type = gconfTypeFromCompType (o->type);
+    GConfValue     *gvalue, *existingValue = NULL;
+    gchar          *key;
 
     GCONF_DISPLAY (d);
 
-    if (plugin)
-    {
-	key = g_strjoin ("/", APP_NAME "/plugins", plugin, screen, "options",
-			 o->name, NULL);
-    }
-    else
-    {
-	key = g_strjoin ("/", APP_NAME "/general", screen, "options", o->name,
-			 NULL);
-    }
+    if (type == GCONF_VALUE_INVALID)
+	return;
 
-    switch (o->type) {
-    case CompOptionTypeBool:
-    case CompOptionTypeInt:
-    case CompOptionTypeFloat:
-    case CompOptionTypeString:
-    case CompOptionTypeColor:
-    case CompOptionTypeMatch:
-	existingValue = gconf_client_get (gd->client, key, NULL);
-	gvalue = gconf_value_new (gconfTypeFromCompType (o->type));
-	gconfSetValue (d, &o->value, o->type, gvalue);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key, gvalue, NULL);
-	gconf_value_free (gvalue);
-	break;
-    case CompOptionTypeAction: {
-	gchar	   *key1, *key2, *key3, *key4, *key5;
+    if (strcmp (plugin, "core") == 0)
+	key = g_strjoin ("/", "/apps", APP_NAME, "general", object, "options",
+			 o->name, NULL);
+    else
+	key = g_strjoin ("/", "/apps", APP_NAME, "plugins", plugin, object,
+			 "options", o->name, NULL);
+
+    existingValue = gconf_client_get (gd->client, key, NULL);
+    gvalue = gconf_value_new (type);
+
+    if (o->type == CompOptionTypeList)
+    {
 	GSList     *node, *list = NULL;
 	GConfValue *gv;
 	int	   i;
 
-	key1 = g_strdup_printf ("%s_%s", key, "key");
-	key2 = g_strdup_printf ("%s_%s", key, "button");
-	key3 = g_strdup_printf ("%s_%s", key, "bell");
-	key4 = g_strdup_printf ("%s_%s", key, "edge");
-	key5 = g_strdup_printf ("%s_%s", key, "edgebutton");
-
-	gvalue = gconf_value_new (GCONF_VALUE_STRING);
-
-	gconfSetActionValue (d, &o->value, o->type, gvalue,
-			     CompBindingTypeKey);
-
-	existingValue = gconf_client_get (gd->client, key1, NULL);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key1, gvalue, NULL);
-
-	if (existingValue)
-	    gconf_value_free (existingValue);
-
-	gconfSetActionValue (d, &o->value, o->type, gvalue,
-			     CompBindingTypeButton);
-
-	existingValue = gconf_client_get (gd->client, key2, NULL);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key2, gvalue, NULL);
-
-	if (existingValue)
-	    gconf_value_free (existingValue);
-
-	gconf_value_free (gvalue);
-
-	gvalue = gconf_value_new (GCONF_VALUE_BOOL);
-	gconf_value_set_bool (gvalue, o->value.action.bell);
-	existingValue = gconf_client_get (gd->client, key3, NULL);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key3, gvalue, NULL);
-
-	if (existingValue)
-	    gconf_value_free (existingValue);
-
-	gconf_value_free (gvalue);
-
-	existingValue = gconf_client_get (gd->client, key4, NULL);
-
-	gvalue = gconf_value_new (GCONF_VALUE_LIST);
-
-	for (i = 0; i < SCREEN_EDGE_NUM; i++)
-	{
-	    if (o->value.action.edgeMask & (1 << i))
-	    {
-		gv = gconf_value_new (GCONF_VALUE_STRING);
-		gconf_value_set_string (gv, edgeToString (i));
-		list = g_slist_append (list, gv);
-	    }
-	}
-
-	gconf_value_set_list_type (gvalue, GCONF_VALUE_STRING);
-	gconf_value_set_list (gvalue, list);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key4, gvalue, NULL);
-
-	for (node = list; node; node = node->next)
-	    gconf_value_free ((GConfValue *) node->data);
-
-	if (existingValue)
-	    gconf_value_free (existingValue);
-
-	g_slist_free (list);
-	gconf_value_free (gvalue);
-
-	gvalue = gconf_value_new (GCONF_VALUE_INT);
-	if (o->value.action.type & CompBindingTypeEdgeButton)
-	    gconf_value_set_int (gvalue, o->value.action.edgeButton);
-	else
-	    gconf_value_set_int (gvalue, 0);
-
-	existingValue = gconf_client_get (gd->client, key5, NULL);
-	if (!existingValue || gconf_value_compare (existingValue, gvalue))
-	    gconf_client_set (gd->client, key5, gvalue, NULL);
-
-	gconf_value_free (gvalue);
-
-	g_free (key1);
-	g_free (key2);
-	g_free (key3);
-	g_free (key4);
-	g_free (key5);
-    } break;
-    case CompOptionTypeList: {
-	GConfValueType type;
-	GSList         *node, *list = NULL;
-	GConfValue     *gv;
-	int	       i;
-
-	existingValue = gconf_client_get (gd->client, key, NULL);
-
-	gvalue = gconf_value_new (GCONF_VALUE_LIST);
-
 	type = gconfTypeFromCompType (o->value.list.type);
 
 	for (i = 0; i < o->value.list.nValue; i++)
@@ -332,6 +211,7 @@ gconfSetOption (CompDisplay *d,
 
 	gconf_value_set_list_type (gvalue, type);
 	gconf_value_set_list (gvalue, list);
+
 	if (!existingValue || gconf_value_compare (existingValue, gvalue))
 	    gconf_client_set (gd->client, key, gvalue, NULL);
 
@@ -339,11 +219,16 @@ gconfSetOption (CompDisplay *d,
 	    gconf_value_free ((GConfValue *) node->data);
 
 	g_slist_free (list);
-	gconf_value_free (gvalue);
-    } break;
-    default:
-	break;
     }
+    else
+    {
+	gconfSetValue (d, &o->value, o->type, gvalue);
+
+	if (!existingValue || gconf_value_compare (existingValue, gvalue))
+	    gconf_client_set (gd->client, key, gvalue, NULL);
+    }
+
+    gconf_value_free (gvalue);
 
     if (existingValue)
 	gconf_value_free (existingValue);
@@ -379,8 +264,15 @@ gconfGetValue (CompDisplay     *d,
     else if (type         == CompOptionTypeString &&
 	     gvalue->type == GCONF_VALUE_STRING)
     {
-	value->s = (char *) gconf_value_get_string (gvalue);
-	return TRUE;
+	const char *str;
+
+	str = gconf_value_get_string (gvalue);
+	if (str)
+	{
+	    value->s = strdup (str);
+	    if (value->s)
+		return TRUE;
+	}
     }
     else if (type         == CompOptionTypeColor &&
 	     gvalue->type == GCONF_VALUE_STRING)
@@ -392,6 +284,42 @@ gconfGetValue (CompDisplay     *d,
 	if (stringToColor (color, value->c))
 	    return TRUE;
     }
+    else if (type         == CompOptionTypeKey &&
+	     gvalue->type == GCONF_VALUE_STRING)
+    {
+	const gchar *action;
+
+	action = gconf_value_get_string (gvalue);
+
+	stringToKeyAction (d, action, &value->action);
+	return TRUE;
+    }
+    else if (type         == CompOptionTypeButton &&
+	     gvalue->type == GCONF_VALUE_STRING)
+    {
+	const gchar *action;
+
+	action = gconf_value_get_string (gvalue);
+
+	stringToButtonAction (d, action, &value->action);
+	return TRUE;
+    }
+    else if (type         == CompOptionTypeEdge &&
+	     gvalue->type == GCONF_VALUE_STRING)
+    {
+	const gchar *edge;
+
+	edge = gconf_value_get_string (gvalue);
+
+	value->action.edgeMask = stringToEdgeMask (edge);
+	return TRUE;
+    }
+    else if (type         == CompOptionTypeBell &&
+	     gvalue->type == GCONF_VALUE_BOOL)
+    {
+	value->action.bell = gconf_value_get_bool (gvalue);
+	return TRUE;
+    }
     else if (type         == CompOptionTypeMatch &&
 	     gvalue->type == GCONF_VALUE_STRING)
     {
@@ -401,7 +329,6 @@ gconfGetValue (CompDisplay     *d,
 
 	matchInit (&value->match);
 	matchAddFromString (&value->match, match);
-
 	return TRUE;
     }
 
@@ -409,413 +336,205 @@ gconfGetValue (CompDisplay     *d,
 }
 
 static Bool
-gconfGetOptionValue (CompDisplay *d,
-		     gchar	 *key)
+gconfReadOptionValue (CompDisplay     *d,
+		      GConfEntry      *entry,
+		      CompOption      *o,
+		      CompOptionValue *value)
 {
-    GConfValue      *gvalue;
-    GConfEntry      *entry;
-    CompOptionValue value;
-    CompPlugin	    *p = 0;
-    CompScreen	    *s = 0;
-    CompOption	    *o, *option;
-    gchar	    *ptr = key;
-    gchar	    *pluginPtr = 0;
-    gint	    pluginLen = 0;
-    gint	    nOption;
-    Bool	    status = FALSE;
+    GConfValue *gvalue;
 
-    GCONF_DISPLAY (d);
-
-    if (strncmp (ptr, APP_NAME, strlen (APP_NAME)))
+    gvalue = gconf_entry_get_value (entry);
+    if (!gvalue)
 	return FALSE;
 
-    ptr += strlen (APP_NAME);
+    compInitOptionValue (value);
 
-    if (strcmpskipifequal (&ptr, "/plugins/") == 0)
+    if (o->type      == CompOptionTypeList &&
+	gvalue->type == GCONF_VALUE_LIST)
     {
-	pluginPtr = ptr;
-	ptr = strchr (ptr, '/');
-	if (!ptr)
+	GConfValueType type;
+	GSList	       *list;
+	int	       i, n;
+
+	type = gconf_value_get_list_type (gvalue);
+	if (gconfTypeFromCompType (o->value.list.type) != type)
 	    return FALSE;
 
-	pluginLen = ptr - pluginPtr;
-	if (pluginLen < 1)
-	    return FALSE;
-    }
-    else if (strcmpskipifequal (&ptr, "/general"))
-	return FALSE;
+	list = gconf_value_get_list (gvalue);
+	n    = g_slist_length (list);
 
-    if (strcmpskipifequal (&ptr, "/screen") == 0)
-    {
-	int screenNum;
-
-	screenNum = strtol (ptr, &ptr, 0);
-
-	for (s = d->screens; s; s = s->next)
-	    if (s->screenNum == screenNum)
-		break;
-
-	if (!s || !ptr)
-	    return FALSE;
-    }
-    else if (strcmpskipifequal (&ptr, "/allscreens"))
-	return FALSE;
-
-    if (strcmpskipifequal (&ptr, "/options/"))
-	return FALSE;
-
-    if (pluginPtr)
-    {
-	pluginPtr = g_strndup (pluginPtr, pluginLen);
-
-	option  = 0;
-	nOption = 0;
-
-	p = findActivePlugin (pluginPtr);
-	if (p)
+	if (n)
 	{
-	    if (s)
+	    value->list.value = malloc (sizeof (CompOptionValue) * n);
+	    if (value->list.value)
 	    {
-		if (p->vTable->getScreenOptions)
-		    option = (*p->vTable->getScreenOptions) (p, s, &nOption);
-	    }
-	    else
-	    {
-		if (p->vTable->getDisplayOptions)
-		    option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
+		for (i = 0; i < n; i++)
+		{
+		    if (!gconfGetValue (d,
+					&value->list.value[i],
+					o->value.list.type,
+					(GConfValue *) list->data))
+			break;
+
+		    value->list.nValue++;
+
+		    list = g_slist_next (list);
+		}
+
+		if (value->list.nValue != n)
+		{
+		    compFiniOptionValue (value, o->type);
+		    return FALSE;
+		}
 	    }
 	}
     }
     else
     {
-	if (s)
-	    option = compGetScreenOptions (s, &nOption);
-	else
-	    option = compGetDisplayOptions (d, &nOption);
+	if (!gconfGetValue (d, value, o->type, gvalue))
+	    return FALSE;
     }
 
-    o = compFindOption (option, nOption, ptr, 0);
-    if (!o)
-    {
-	if (pluginPtr)
-	    g_free (pluginPtr);
+    gconf_entry_free (entry);
 
-	return FALSE;
-    }
-
-    if (o->type == CompOptionTypeAction)
-    {
-	gchar *key1, *key2, *key3, *key4, *key5;
-
-	key1 = g_strdup_printf ("%s_%s", key, "key");
-	key2 = g_strdup_printf ("%s_%s", key, "button");
-	key3 = g_strdup_printf ("%s_%s", key, "bell");
-	key4 = g_strdup_printf ("%s_%s", key, "edge");
-	key5 = g_strdup_printf ("%s_%s", key, "edgebutton");
-
-	value = o->value;
-
-	entry = gconf_client_get_entry (gd->client, key1, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue && gvalue->type == GCONF_VALUE_STRING)
-	    {
-		const char *binding;
-
-		binding = gconf_value_get_string (gvalue);
-		if (!binding)
-		    binding = "";
-
-		if (strcasecmp (binding, "disabled") == 0 || !*binding)
-		{
-		    value.action.type &= ~CompBindingTypeKey;
-		    status = TRUE;
-		}
-		else
-		{
-		    value.action.type |= CompBindingTypeKey;
-		    status = stringToKeyBinding (d, binding,
-						 &value.action.key);
-		}
-	    }
-
-	    gconf_entry_free (entry);
-	}
-
-	entry = gconf_client_get_entry (gd->client, key2, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue && gvalue->type == GCONF_VALUE_STRING)
-	    {
-		const char *binding;
-
-		binding = gconf_value_get_string (gvalue);
-		if (!binding)
-		    binding = "";
-
-		if (strcasecmp (binding, "disabled") == 0 || !*binding)
-		{
-		    value.action.type &= ~CompBindingTypeButton;
-		    status = TRUE;
-		}
-		else
-		{
-		    value.action.type |= CompBindingTypeButton;
-		    status = stringToButtonBinding (d, binding,
-						    &value.action.button);
-		}
-	    }
-
-	    gconf_entry_free (entry);
-	}
-
-	entry = gconf_client_get_entry (gd->client, key3, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue && gvalue->type == GCONF_VALUE_BOOL)
-	    {
-		value.action.bell = gconf_value_get_bool (gvalue);
-		status = TRUE;
-	    }
-
-	    gconf_entry_free (entry);
-	}
-
-	entry = gconf_client_get_entry (gd->client, key4, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue && gvalue->type == GCONF_VALUE_LIST)
-	    {
-		if (gconf_value_get_list_type (gvalue) == GCONF_VALUE_STRING)
-		{
-		    GSList *list;
-		    gchar  *edge;
-		    int    i;
-
-		    value.action.edgeMask = 0;
-
-		    status = TRUE;
-
-		    list = gconf_value_get_list (gvalue);
-		    while (list)
-		    {
-			GConfValue *gv;
-
-			gv = (GConfValue *) list->data;
-			edge = (gchar *) gconf_value_get_string (gv);
-
-			for (i = 0; i < SCREEN_EDGE_NUM; i++)
-			{
-			    if (strcasecmp (edge, edgeToString (i)) == 0)
-				value.action.edgeMask |= 1 << i;
-			}
-
-			list = g_slist_next (list);
-		    }
-		}
-	    }
-
-	    gconf_entry_free (entry);
-	}
-
-	entry = gconf_client_get_entry (gd->client, key5, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue && gvalue->type == GCONF_VALUE_INT)
-	    {
-		value.action.edgeButton = gconf_value_get_int (gvalue);
-
-		if (value.action.edgeButton)
-		    value.action.type |= CompBindingTypeEdgeButton;
-		else
-		    value.action.type &= ~CompBindingTypeEdgeButton;
-
-		status = TRUE;
-	    }
-
-	    gconf_entry_free (entry);
-	}
-
-	if (status)
-	{
-	    if (s)
-	    {
-		if (pluginPtr)
-		    status = (*s->setScreenOptionForPlugin) (s,
-							     pluginPtr,
-							     ptr,
-							     &value);
-		else
-		    status = (*s->setScreenOption) (s, ptr, &value);
-	    }
-	    else
-	    {
-		if (pluginPtr)
-		    status = (*d->setDisplayOptionForPlugin) (d,
-							      pluginPtr,
-							      ptr,
-							      &value);
-		else
-		    status = (*d->setDisplayOption) (d, ptr, &value);
-	    }
-	}
-
-	g_free (key1);
-	g_free (key2);
-	g_free (key3);
-	g_free (key4);
-	g_free (key5);
-    }
-    else
-    {
-	entry = gconf_client_get_entry (gd->client, key, NULL, TRUE, NULL);
-	if (entry)
-	{
-	    gvalue = gconf_entry_get_value (entry);
-	    if (gvalue)
-	    {
-		if (o->type      == CompOptionTypeList &&
-		    gvalue->type == GCONF_VALUE_LIST)
-		{
-		    GConfValueType type;
-
-		    value.list.value  = 0;
-		    value.list.nValue = 0;
-
-		    type = gconf_value_get_list_type (gvalue);
-		    if (type == gconfTypeFromCompType (o->value.list.type))
-		    {
-			GSList *list;
-			int    i, length;
-
-			status = TRUE;
-
-			list = gconf_value_get_list (gvalue);
-
-			length = g_slist_length (list);
-
-			if (length)
-			{
-			    value.list.value =
-				malloc (sizeof (CompOptionValue) * length);
-			    if (value.list.value)
-			    {
-				for (i = 0; i < length; i++)
-				{
-				    if (!gconfGetValue (d, &value.list.value[i],
-							o->value.list.type,
-							(GConfValue *)
-							list->data))
-				    {
-					status = FALSE;
-					break;
-				    }
-
-				    value.list.nValue++;
-
-				    list = g_slist_next (list);
-				}
-			    }
-			    else
-				status = FALSE;
-
-			    if (!status)
-			    {
-				free (value.list.value);
-			    }
-			}
-		    }
-		}
-		else
-		{
-		    status = gconfGetValue (d, &value, o->type, gvalue);
-		}
-
-		if (status)
-		{
-		    if (s)
-		    {
-			if (pluginPtr)
-			    status = (*s->setScreenOptionForPlugin) (s,
-								     pluginPtr,
-								     ptr,
-								     &value);
-			else
-			    status = (*s->setScreenOption) (s, ptr, &value);
-		    }
-		    else
-		    {
-			if (pluginPtr)
-			    status = (*d->setDisplayOptionForPlugin) (d,
-								      pluginPtr,
-								      ptr,
-								      &value);
-			else
-			    status = (*d->setDisplayOption) (d, ptr, &value);
-		    }
-
-		    if (o->type == CompOptionTypeList)
-		    {
-			if (o->value.list.type == CompOptionTypeMatch)
-			{
-			    int i;
-
-			    for (i = 0; i < value.list.nValue; i++)
-				matchFini (&value.list.value[i].match);
-			}
-
-			if (value.list.value)
-			    free (value.list.value);
-		    }
-		    else if (o->type == CompOptionTypeMatch)
-		    {
-			matchFini (&value.match);
-		    }
-		}
-	    }
-
-	    gconf_entry_free (entry);
-	}
-    }
-
-    if (pluginPtr)
-	g_free (pluginPtr);
-
-    return status;
+    return TRUE;
 }
 
 static void
-gconfInitOption (CompDisplay *d,
-		 CompOption  *o,
-		 gchar       *screen,
-		 gchar       *plugin)
+gconfGetDisplayOption (CompDisplay *d,
+		       CompOption  *o,
+		       const char  *plugin)
 {
-    gchar *key;
+    GConfEntry *entry;
+    gchar      *key;
 
-    if (plugin)
-    {
-	key = g_strjoin ("/", APP_NAME "/plugins", plugin, screen,
+    GCONF_DISPLAY (d);
+
+    if (strcmp (plugin, "core") == 0)
+	key = g_strjoin ("/", "/apps", APP_NAME, "general", "allscreens",
 			 "options", o->name, NULL);
-    }
     else
+	key = g_strjoin ("/", "/apps", APP_NAME, "plugins", plugin,
+			 "allscreens", "options", o->name, NULL);
+
+    entry = gconf_client_get_entry (gd->client, key, NULL, TRUE, NULL);
+    if (entry)
     {
-	key = g_strjoin ("/", APP_NAME "/general", screen, "options",
-			 o->name, NULL);
+	CompOptionValue value;
+
+	if (gconfReadOptionValue (d, entry, o, &value))
+	{
+	    if (strcmp (plugin, "core") == 0)
+		(*d->setDisplayOption) (d, o->name, &value);
+	    else
+		(*d->setDisplayOptionForPlugin) (d, plugin, o->name, &value);
+
+	    compFiniOptionValue (&value, o->type);
+	}
+	else
+	{
+	    gconfSetOption (d, o, plugin, "allscreens");
+	}
     }
 
-    gconfGetOptionValue (d, key);
+    g_free (key);
+}
+
+static void
+gconfGetScreenOption (CompScreen *s,
+		      CompOption *o,
+		      const char *plugin,
+		      const char *screen)
+{
+    GConfEntry *entry;
+    gchar      *key;
+
+    GCONF_DISPLAY (s->display);
+
+    if (strcmp (plugin, "core") == 0)
+	key = g_strjoin ("/", "/apps", APP_NAME, "general", screen, "options",
+			 o->name, NULL);
+    else
+	key = g_strjoin ("/", "/apps", APP_NAME, "plugins", plugin, screen,
+			 "options", o->name, NULL);
+
+    entry = gconf_client_get_entry (gd->client, key, NULL, TRUE, NULL);
+    if (entry)
+    {
+	CompOptionValue value;
+
+	if (gconfReadOptionValue (s->display, entry, o, &value))
+	{
+	    if (strcmp (plugin, "core") == 0)
+		(*s->setScreenOption) (s, o->name, &value);
+	    else
+		(*s->setScreenOptionForPlugin) (s, plugin, o->name, &value);
+
+	    compFiniOptionValue (&value, o->type);
+	}
+	else
+	{
+	    gconfSetOption (s->display, o, plugin, screen);
+	}
+    }
 
     g_free (key);
 }
 
+static Bool
+gconfReload (void *closure)
+{
+    CompDisplay *d = (CompDisplay *) closure;
+    CompScreen  *s;
+    CompPlugin  *p;
+    CompOption  *option;
+    int		nOption;
+
+    GCONF_DISPLAY (d);
+
+    option = compGetDisplayOptions (d, &nOption);
+    while (nOption--)
+	gconfGetDisplayOption (d, option++, "core");
+
+    for (p = getPlugins (); p; p = p->next)
+    {
+	if (!p->vTable->getDisplayOptions)
+	    continue;
+
+	option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
+	while (nOption--)
+	    gconfGetDisplayOption (d, option++, p->vTable->name);
+    }
+
+    for (s = d->screens; s; s = s->next)
+    {
+	gchar *screen = g_strdup_printf ("screen%d", s->screenNum);
+
+	option = compGetScreenOptions (s, &nOption);
+	while (nOption--)
+	    gconfGetScreenOption (s, option++, "core", screen);
+
+	for (p = getPlugins (); p; p = p->next)
+	{
+	    if (!p->vTable->getScreenOptions)
+		continue;
+
+	    option = (*p->vTable->getScreenOptions) (p, s, &nOption);
+	    while (nOption--)
+		gconfGetScreenOption (s, option++, p->vTable->name, screen);
+	}
+
+	g_free (screen);
+    }
+
+    gd->reloadHandle = 0;
+
+    return FALSE;
+}
+
 static Bool
 gconfSetDisplayOption (CompDisplay     *d,
-		       char	       *name,
+		       const char      *name,
 		       CompOptionValue *value)
 {
     Bool status;
@@ -826,14 +545,15 @@ gconfSetDisplayOption (CompDisplay     *d,
     status = (*d->setDisplayOption) (d, name, value);
     WRAP (gd, d, setDisplayOption, gconfSetDisplayOption);
 
-    if (status)
+    if (status && !gd->reloadHandle)
     {
 	CompOption *option;
 	int	   nOption;
 
 	option = compGetDisplayOptions (d, &nOption);
-	gconfSetOption (d, compFindOption (option, nOption, name, 0),
-			"allscreens", 0);
+	option = compFindOption (option, nOption, name, 0);
+	if (option)
+	    gconfSetOption (d, option, "core", "allscreens");
     }
 
     return status;
@@ -841,8 +561,8 @@ gconfSetDisplayOption (CompDisplay     *d,
 
 static Bool
 gconfSetDisplayOptionForPlugin (CompDisplay     *d,
-				char	        *plugin,
-				char	        *name,
+				const char	*plugin,
+				const char	*name,
 				CompOptionValue *value)
 {
     Bool status;
@@ -853,7 +573,7 @@ gconfSetDisplayOptionForPlugin (CompDisplay     *d,
     status = (*d->setDisplayOptionForPlugin) (d, plugin, name, value);
     WRAP (gd, d, setDisplayOptionForPlugin, gconfSetDisplayOptionForPlugin);
 
-    if (status)
+    if (status && !gd->reloadHandle)
     {
 	CompPlugin *p;
 
@@ -864,8 +584,9 @@ gconfSetDisplayOptionForPlugin (CompDisplay     *d,
 	    int	       nOption;
 
 	    option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
-	    gconfSetOption (d, compFindOption (option, nOption, name, 0),
-			    "allscreens", plugin);
+	    option = compFindOption (option, nOption, name, 0);
+	    if (option)
+		gconfSetOption (d, option, p->vTable->name, "allscreens");
 	}
     }
 
@@ -874,7 +595,7 @@ gconfSetDisplayOptionForPlugin (CompDisplay     *d,
 
 static Bool
 gconfSetScreenOption (CompScreen      *s,
-		      char	      *name,
+		      const char      *name,
 		      CompOptionValue *value)
 {
     Bool status;
@@ -887,17 +608,23 @@ gconfSetScreenOption (CompScreen      *s,
 
     if (status)
     {
-	CompOption *option;
-	int	   nOption;
-	gchar      *screen;
+	GCONF_DISPLAY (s->display);
 
-	screen = g_strdup_printf ("screen%d", s->screenNum);
+	if (!gd->reloadHandle)
+	{
+	    CompOption *option;
+	    int	       nOption;
+	    gchar      *screen;
 
-	option = compGetScreenOptions (s, &nOption);
-	gconfSetOption (s->display, compFindOption (option, nOption, name, 0),
-			screen, 0);
+	    screen = g_strdup_printf ("screen%d", s->screenNum);
 
-	g_free (screen);
+	    option = compGetScreenOptions (s, &nOption);
+	    option = compFindOption (option, nOption, name, 0);
+	    if (option)
+		gconfSetOption (s->display, option, "core", screen);
+
+	    g_free (screen);
+	}
     }
 
     return status;
@@ -905,8 +632,8 @@ gconfSetScreenOption (CompScreen      *s,
 
 static Bool
 gconfSetScreenOptionForPlugin (CompScreen      *s,
-			       char	       *plugin,
-			       char	       *name,
+			       const char      *plugin,
+			       const char      *name,
 			       CompOptionValue *value)
 {
     Bool status;
@@ -919,23 +646,28 @@ gconfSetScreenOptionForPlugin (CompScreen      *s,
 
     if (status)
     {
-	CompPlugin *p;
+	GCONF_DISPLAY (s->display);
 
-	p = findActivePlugin (plugin);
-	if (p && p->vTable->getScreenOptions)
+	if (!gd->reloadHandle)
 	{
-	    CompOption *option;
-	    int	       nOption;
-	    gchar      *screen;
+	    CompPlugin *p;
 
-	    screen = g_strdup_printf ("screen%d", s->screenNum);
+	    p = findActivePlugin (plugin);
+	    if (p && p->vTable->getScreenOptions)
+	    {
+		CompOption *option;
+		int	   nOption;
+		gchar      *screen;
 
-	    option = (*p->vTable->getScreenOptions) (p, s, &nOption);
-	    gconfSetOption (s->display,
-			    compFindOption (option, nOption, name, 0),
-			    screen, plugin);
+		screen = g_strdup_printf ("screen%d", s->screenNum);
 
-	    g_free (screen);
+		option = compGetScreenOptions (s, &nOption);
+		option = compFindOption (option, nOption, name, 0);
+		if (option)
+		    gconfSetOption (s->display, option, plugin, screen);
+
+		g_free (screen);
+	    }
 	}
     }
 
@@ -961,7 +693,7 @@ gconfInitPluginForDisplay (CompPlugin  *p,
 
 	option = (*p->vTable->getDisplayOptions) (p, d, &nOption);
 	while (nOption--)
-	    gconfInitOption (d, option++, "allscreens", p->vTable->name);
+	    gconfGetDisplayOption (d, option++, p->vTable->name);
     }
 
     return status;
@@ -989,7 +721,7 @@ gconfInitPluginForScreen (CompPlugin *p,
 
 	option = (*p->vTable->getScreenOptions) (p, s, &nOption);
 	while (nOption--)
-	    gconfInitOption (s->display, option++, screen, p->vTable->name);
+	    gconfGetScreenOption (s, option++, p->vTable->name, screen);
 
 	g_free (screen);
     }
@@ -1003,23 +735,143 @@ gconfKeyChanged (GConfClient *client,
 		 GConfEntry  *entry,
 		 gpointer    user_data)
 {
-    static int  tail[] = { 0, 4, 5, 7, 11 };
     CompDisplay *display = (CompDisplay *) user_data;
-    gchar	*key = g_strdup (entry->key);
-    int		i;
+    CompScreen  *screen = NULL;
+    CompPlugin  *plugin = NULL;
+    CompOption  *option = NULL;
+    int		nOption = 0;
+    gchar	**token;
+    int		object = 4;
 
-    for (i = 0; i < sizeof (tail) / sizeof (tail[0]); i++)
+    token = g_strsplit (entry->key, "/", 8);
+
+    if (g_strv_length (token) < 7)
     {
-	if (strlen (entry->key) > tail[i])
+	g_strfreev (token);
+	return;
+    }
+
+    if (strcmp (token[0], "")	    != 0 ||
+	strcmp (token[1], "apps")   != 0 ||
+	strcmp (token[2], APP_NAME) != 0)
+    {
+	g_strfreev (token);
+	return;
+    }
+
+    if (strcmp (token[3], "general") != 0)
+    {
+	if (strcmp (token[3], "plugins") != 0 || g_strv_length (token) < 8)
 	{
-	    key[strlen (entry->key) - tail[i]] = '\0';
-	    if (gconfGetOptionValue (display, key))
+	    g_strfreev (token);
+	    return;
+	}
+
+	plugin = findActivePlugin (token[4]);
+	if (!plugin)
+	{
+	    g_strfreev (token);
+	    return;
+	}
+
+	object = 5;
+    }
+
+    if (strcmp (token[object], "allscreens") != 0)
+    {
+	int screenNum;
+
+	if (sscanf (token[object], "screen%d", &screenNum) != 1)
+	{
+	    g_strfreev (token);
+	    return;
+	}
+
+	for (screen = display->screens; screen; screen = screen->next)
+	    if (screen->screenNum == screenNum)
 		break;
+
+	if (!screen)
+	{
+	    g_strfreev (token);
+	    return;
 	}
     }
 
-    if (key)
-	g_free (key);
+    if (strcmp (token[object + 1], "options") != 0)
+    {
+	g_strfreev (token);
+	return;
+    }
+
+    if (screen)
+    {
+	if (plugin)
+	{
+	    if (plugin->vTable->getScreenOptions)
+		option = (*plugin->vTable->getScreenOptions) (plugin, screen,
+							      &nOption);
+	}
+	else
+	{
+	    option = compGetScreenOptions (screen, &nOption);
+	}
+
+	option = compFindOption (option, nOption, token[object + 2], 0);
+	if (option)
+	{
+	    CompOptionValue value;
+
+	    if (gconfReadOptionValue (display, entry, option, &value))
+	    {
+		if (plugin)
+		    (*screen->setScreenOptionForPlugin) (screen,
+							 token[4],
+							 option->name,
+							 &value);
+		else
+		    (*screen->setScreenOption) (screen, option->name, &value);
+
+		compFiniOptionValue (&value, option->type);
+	    }
+	}
+    }
+    else
+    {
+	if (plugin)
+	{
+	    if (plugin->vTable->getDisplayOptions)
+		option = (*plugin->vTable->getDisplayOptions) (plugin, display,
+							       &nOption);
+	}
+	else
+	{
+	    option = compGetDisplayOptions (display, &nOption);
+	}
+
+	option = compFindOption (option, nOption, token[object + 2], 0);
+	if (option)
+	{
+	    CompOptionValue value;
+
+	    if (gconfReadOptionValue (display, entry, option, &value))
+	    {
+		if (plugin)
+		    (*display->setDisplayOptionForPlugin) (display,
+							   token[4],
+							   option->name,
+							   &value);
+		else
+		    (*display->setDisplayOption) (display,
+						  option->name,
+						  &value);
+
+		compFiniOptionValue (&value, option->type);
+	    }
+	}
+    }
+
+    g_strfreev (token);
 }
 
 static void
@@ -1048,8 +900,6 @@ static Bool
 gconfInitDisplay (CompPlugin  *p,
 		  CompDisplay *d)
 {
-    CompOption   *option;
-    int	         nOption;
     GConfDisplay *gd;
 
     gd = malloc (sizeof (GConfDisplay));
@@ -1067,21 +917,19 @@ gconfInitDisplay (CompPlugin  *p,
 
     gd->client = gconf_client_get_default ();
 
-    gconf_client_add_dir (gd->client, APP_NAME,
+    gconf_client_add_dir (gd->client, "/apps/" APP_NAME,
 			  GCONF_CLIENT_PRELOAD_NONE, NULL);
 
+    gd->reloadHandle = compAddTimeout (0, gconfReload, (void *) d);
+
     WRAP (gd, d, initPluginForDisplay, gconfInitPluginForDisplay);
     WRAP (gd, d, setDisplayOption, gconfSetDisplayOption);
     WRAP (gd, d, setDisplayOptionForPlugin, gconfSetDisplayOptionForPlugin);
 
     d->privates[displayPrivateIndex].ptr = gd;
 
-    option = compGetDisplayOptions (d, &nOption);
-    while (nOption--)
-	gconfInitOption (d, option++, "allscreens", 0);
-
-    gconf_client_notify_add (gd->client, APP_NAME, gconfKeyChanged, d,
-			     NULL, NULL);
+    gconf_client_notify_add (gd->client, "/apps/" APP_NAME, gconfKeyChanged,
+			     d, NULL, NULL);
 
     gconfSendGLibNotify (d);
 
@@ -1094,6 +942,9 @@ gconfFiniDisplay (CompPlugin  *p,
 {
     GCONF_DISPLAY (d);
 
+    if (gd->reloadHandle)
+	compRemoveTimeout (gd->reloadHandle);
+
     g_object_unref (gd->client);
 
     UNWRAP (gd, d, initPluginForDisplay);
@@ -1109,10 +960,7 @@ static Bool
 gconfInitScreen (CompPlugin *p,
 		 CompScreen *s)
 {
-    CompOption  *option;
-    int	        nOption;
     GConfScreen *gs;
-    gchar       *screen;
 
     GCONF_DISPLAY (s->display);
 
@@ -1126,14 +974,6 @@ gconfInitScreen (CompPlugin *p,
 
     s->privates[gd->screenPrivateIndex].ptr = gs;
 
-    screen = g_strdup_printf ("screen%d", s->screenNum);
-
-    option = compGetScreenOptions (s, &nOption);
-    while (nOption--)
-	gconfInitOption (s->display, option++, screen, 0);
-
-    g_free (screen);
-
     return TRUE;
 }
 
