@@ -29,12 +29,12 @@
 #include <math.h>
 #include <unistd.h>
 
-#include <X11/Xatom.h>
-#include <X11/extensions/shape.h>
-
 #include <compiz.h>
 #include <decoration.h>
 
+#include <X11/Xatom.h>
+#include <X11/extensions/shape.h>
+
 /*
  * compiz composited video
  *
@@ -207,9 +207,9 @@ videoGetDisplayOptions (CompPlugin  *plugin,
 }
 
 static Bool
-videoSetDisplayOption (CompPlugin  *plugin,
+videoSetDisplayOption (CompPlugin      *plugin,
 		       CompDisplay     *display,
-		       char	       *name,
+		       const char      *name,
 		       CompOptionValue *value)
 {
     CompOption *o;
