@@ -467,13 +467,13 @@ runCommandDispatch (CompDisplay     *d,
     if (s)
     {
 	int index = -1;
-	int i = COMP_DISPLAY_OPTION_RUN_COMMAND0;
+	int i = COMP_DISPLAY_OPTION_RUN_COMMAND0_KEY;
 
-	while (i <= COMP_DISPLAY_OPTION_RUN_COMMAND11)
+	while (i <= COMP_DISPLAY_OPTION_RUN_COMMAND11_KEY)
 	{
 	    if (action == &d->opt[i].value.action)
 	    {
-		index = i - COMP_DISPLAY_OPTION_RUN_COMMAND0 +
+		index = i - COMP_DISPLAY_OPTION_RUN_COMMAND0_KEY +
 		    COMP_DISPLAY_OPTION_COMMAND0;
 		break;
 	    }
@@ -670,9 +670,9 @@ const CompMetadataOptionInfo coreDisplayOptionInfo[COMP_DISPLAY_OPTION_NUM] = {
     { "click_to_focus", "bool", 0, 0, 0 },
     { "autoraise", "bool", 0, 0, 0 },
     { "autoraise_delay", "int", 0, 0, 0 },
-    { "close_window", "action", 0, closeWin, 0 },
-    { "main_menu", "action", 0, mainMenu, 0 },
-    { "run", "action", 0, runDialog, 0 },
+    { "close_window_key", "key", 0, closeWin, 0 },
+    { "main_menu_key", "key", 0, mainMenu, 0 },
+    { "run_key", "key", 0, runDialog, 0 },
     { "command0", "string", 0, 0, 0 },
     { "command1", "string", 0, 0, 0 },
     { "command2", "string", 0, 0, 0 },
@@ -685,47 +685,48 @@ const CompMetadataOptionInfo coreDisplayOptionInfo[COMP_DISPLAY_OPTION_NUM] = {
     { "command9", "string", 0, 0, 0 },
     { "command10", "string", 0, 0, 0 },
     { "command11", "string", 0, 0, 0 },
-    { "run_command0", "action", 0, runCommandDispatch, 0 },
-    { "run_command1", "action", 0, runCommandDispatch, 0 },
-    { "run_command2", "action", 0, runCommandDispatch, 0 },
-    { "run_command3", "action", 0, runCommandDispatch, 0 },
-    { "run_command4", "action", 0, runCommandDispatch, 0 },
-    { "run_command5", "action", 0, runCommandDispatch, 0 },
-    { "run_command6", "action", 0, runCommandDispatch, 0 },
-    { "run_command7", "action", 0, runCommandDispatch, 0 },
-    { "run_command8", "action", 0, runCommandDispatch, 0 },
-    { "run_command9", "action", 0, runCommandDispatch, 0 },
-    { "run_command10", "action", 0, runCommandDispatch, 0 },
-    { "run_command11", "action", 0, runCommandDispatch, 0 },
-    { "slow_animations", "action", 0, toggleSlowAnimations, 0 },
-    { "raise_window", "action", 0, raiseInitiate, 0 },
-    { "lower_window", "action", 0, lowerInitiate, 0 },
-    { "unmaximize_window", "action", 0, unmaximize, 0 },
-    { "minimize_window", "action", 0, minimize, 0 },
-    { "maximize_window", "action", 0, maximize, 0 },
-    { "maximize_window_horizontally", "action", 0, maximizeHorizontally, 0 },
-    { "maximize_window_vertically", "action", 0, maximizeVertically, 0 },
-    { "opacity_increase", "action", 0, increaseOpacity, 0 },
-    { "opacity_decrease", "action", 0, decreaseOpacity, 0 },
+    { "run_command0_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command1_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command2_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command3_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command4_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command5_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command6_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command7_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command8_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command9_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command10_key", "key", 0, runCommandDispatch, 0 },
+    { "run_command11_key", "key", 0, runCommandDispatch, 0 },
+    { "slow_animations_key", "key", 0, toggleSlowAnimations, 0 },
+    { "raise_window_button", "button", 0, raiseInitiate, 0 },
+    { "lower_window_button", "button", 0, lowerInitiate, 0 },
+    { "unmaximize_window_key", "key", 0, unmaximize, 0 },
+    { "minimize_window_key", "key", 0, minimize, 0 },
+    { "maximize_window_key", "key", 0, maximize, 0 },
+    { "maximize_window_horizontally_key", "key", 0, maximizeHorizontally, 0 },
+    { "maximize_window_vertically_key", "key", 0, maximizeVertically, 0 },
+    { "opacity_increase_button", "button", 0, increaseOpacity, 0 },
+    { "opacity_decrease_button", "button", 0, decreaseOpacity, 0 },
     { "command_screenshot", "string", 0, 0, 0 },
-    { "run_command_screenshot", "action", 0, runCommandScreenshot, 0 },
+    { "run_command_screenshot_key", "key", 0, runCommandScreenshot, 0 },
     { "command_window_screenshot", "string", 0, 0, 0 },
-    { "run_command_window_screenshot", "action", 0,
+    { "run_command_window_screenshot_key", "key", 0,
       runCommandWindowScreenshot, 0 },
-    { "window_menu", "action", 0, windowMenu, 0 },
-    { "show_desktop", "action", 0, showDesktop, 0 },
+    { "window_menu_button", "button", 0, windowMenu, 0 },
+    { "window_menu_key", "key", 0, windowMenu, 0 },
+    { "show_desktop_key", "key", 0, showDesktop, 0 },
     { "raise_on_click", "bool", 0, 0, 0 },
     { "audible_bell", "bool", 0, 0, 0 },
-    { "toggle_window_maximized", "action", 0, toggleMaximized, 0 },
-    { "toggle_window_maximized_horizontally", "action", 0,
+    { "toggle_window_maximized_key", "key", 0, toggleMaximized, 0 },
+    { "toggle_window_maximized_horizontally_key", "key", 0,
       toggleMaximizedHorizontally, 0 },
-    { "toggle_window_maximized_vertically", "action", 0,
+    { "toggle_window_maximized_vertically_key", "key", 0,
       toggleMaximizedVertically, 0 },
     { "hide_skip_taskbar_windows", "bool", 0, 0, 0 },
-    { "toggle_window_shaded", "action", 0, shade, 0 },
+    { "toggle_window_shaded_key", "key", 0, shade, 0 },
     { "ignore_hints_when_maximized", "bool", 0, 0, 0 },
     { "command_terminal", "string", 0, 0, 0 },
-    { "run_command_terminal", "action", 0, runCommandTerminal, 0 },
+    { "run_command_terminal_key", "key", 0, runCommandTerminal, 0 },
     { "ping_delay", "int", "<min>1000</min>", 0, 0 }
 };
 
@@ -822,7 +823,7 @@ pingTimeout (void *closure)
 
 static Bool
 setDisplayOption (CompDisplay     *display,
-		  char	          *name,
+		  const char	  *name,
 		  CompOptionValue *value)
 {
     CompOption *o;
@@ -885,8 +886,8 @@ setDisplayOption (CompDisplay     *display,
 
 static Bool
 setDisplayOptionForPlugin (CompDisplay     *display,
-			   char	           *plugin,
-			   char	           *name,
+			   const char	   *plugin,
+			   const char	   *name,
 			   CompOptionValue *value)
 {
     CompPlugin *p;
@@ -1431,11 +1432,6 @@ waitForVideoSync (CompScreen *s)
     if (!s->opt[COMP_SCREEN_OPTION_SYNC_TO_VBLANK].value.b)
 	return;
 
-    /* we currently can't handle sync to vblank when we have more than one
-       output device */
-    if (s->nOutputDev > 1)
-	return;
-
     if (s->getVideoSync)
     {
 	glFlush ();
@@ -1932,11 +1928,11 @@ addScreenActions (CompScreen *s)
 
     for (i = 0; i < COMP_DISPLAY_OPTION_NUM; i++)
     {
-	if (s->display->opt[i].type == CompOptionTypeAction)
-	{
-	    if (s->display->opt[i].value.action.state & CompActionStateAutoGrab)
-		addScreenAction (s, &s->display->opt[i].value.action);
-	}
+	if (!isActionOption (&s->display->opt[i]))
+	    continue;
+
+	if (s->display->opt[i].value.action.state & CompActionStateAutoGrab)
+	    addScreenAction (s, &s->display->opt[i].value.action);
     }
 }
 
@@ -1957,7 +1953,7 @@ addScreenToDisplay (CompDisplay *display,
 }
 
 Bool
-addDisplay (char *name)
+addDisplay (const char *name)
 {
     CompDisplay *d;
     Display     *dpy;
@@ -2015,6 +2011,8 @@ addDisplay (char *name)
 	return FALSE;
     }
 
+    d->connection = XGetXCBConnection (dpy);
+
     if (!compInitDisplayOptionsFromMetadata (d,
 					     &coreMetadata,
 					     coreDisplayOptionInfo,
@@ -2664,21 +2662,14 @@ CompWindow *
 findWindowAtDisplay (CompDisplay *d,
 		     Window      id)
 {
-    if (lastFoundWindow && lastFoundWindow->id == id)
-    {
-	return lastFoundWindow;
-    }
-    else
-    {
-	CompScreen *s;
-	CompWindow *w;
+    CompScreen *s;
+    CompWindow *w;
 
-	for (s = d->screens; s; s = s->next)
-	{
-	    w = findWindowAtScreen (s, id);
-	    if (w)
-		return w;
-	}
+    for (s = d->screens; s; s = s->next)
+    {
+	w = findWindowAtScreen (s, id);
+	if (w)
+	    return w;
     }
 
     return 0;
@@ -2688,21 +2679,14 @@ CompWindow *
 findTopLevelWindowAtDisplay (CompDisplay *d,
 			     Window      id)
 {
-    if (lastFoundWindow && lastFoundWindow->id == id)
-    {
-	return lastFoundWindow;
-    }
-    else
-    {
-	CompScreen *s;
-	CompWindow *w;
+    CompScreen *s;
+    CompWindow *w;
 
-	for (s = d->screens; s; s = s->next)
-	{
-	    w = findTopLevelWindowAtScreen (s, id);
-	    if (w)
-		return w;
-	}
+    for (s = d->screens; s; s = s->next)
+    {
+	w = findTopLevelWindowAtScreen (s, id);
+	if (w)
+	    return w;
     }
 
     return 0;
