@@ -198,15 +198,15 @@ autoRaiseTimeout (void *closure)
     return FALSE;
 }
 
-#define REAL_MOD_MASK (ShiftMask | ControlMask | Mod1Mask | \
-		       Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)
+#define REAL_MOD_MASK (ShiftMask | ControlMask | Mod1Mask | Mod2Mask | \
+		       Mod3Mask | Mod4Mask | Mod5Mask | CompNoMask)
 
 static Bool
 isCallBackBinding (CompOption	   *option,
 		   CompBindingType type,
 		   CompActionState state)
 {
-    if (option->type != CompOptionTypeAction)
+    if (!isActionOption (option))
 	return FALSE;
 
     if (!(option->value.action.type & type))
@@ -312,9 +312,14 @@ triggerButtonPressBindings (CompDisplay *d,
 	    if (isInitiateBinding (option, CompBindingTypeEdgeButton,
 				   CompActionStateInitEdge, &action))
 	    {
-		if (action->edgeMask & edge)
+		if ((action->button.button == event->xbutton.button) &&
+		    (action->edgeMask & edge))
 		{
-		    if (action->edgeButton == event->xbutton.button)
+		    bindMods = virtualToRealModMask (d,
+						     action->button.modifiers);
+
+		    if ((bindMods & modMask) ==
+			(event->xbutton.state & modMask))
 			if ((*action->initiate) (d, action,
 						 CompActionStateInitEdge,
 						 argument, nArgument))
@@ -383,7 +388,7 @@ triggerKeyPressBindings (CompDisplay *d,
 
 	while (n--)
 	{
-	    if (o->type == CompOptionTypeAction)
+	    if (isActionOption (o))
 	    {
 		if (o->value.action.terminate)
 		    (*o->value.action.terminate) (d, &o->value.action,
@@ -534,7 +539,8 @@ isBellAction (CompOption      *option,
 	      CompActionState state,
 	      CompAction      **action)
 {
-    if (option->type != CompOptionTypeAction)
+    if (option->type != CompOptionTypeAction &&
+	option->type != CompOptionTypeBell)
 	return FALSE;
 
     if (!option->value.action.bell)
@@ -580,7 +586,9 @@ isEdgeAction (CompOption      *option,
 	      CompActionState state,
 	      unsigned int    edge)
 {
-    if (option->type != CompOptionTypeAction)
+    if (option->type != CompOptionTypeAction &&
+	option->type != CompOptionTypeButton &&
+	option->type != CompOptionTypeEdge)
 	return FALSE;
 
     if (!(option->value.action.edgeMask & edge))
@@ -1136,8 +1144,8 @@ handleActionEvent (CompDisplay *d,
 
 void
 handleCompizEvent (CompDisplay *d,
-		   char        *pluginName,
-		   char        *eventName,
+		   const char  *pluginName,
+		   const char  *eventName,
 		   CompOption  *option,
 		   int         nOption)
 {
